# OwnCloud

- Author: Atul Jha <atul.jha@csscorp.com>
- Maintainer: José Antonio Rey <jose@ubuntu.com>

# Overview

ownCloud provides universal access to your files via the web, your computer or
your mobile devices — wherever you are. It also provides a platform to easily
view & sync your contacts, calendars and bookmarks across all your devices and
enables basic editing right on the web.

This charm installs Apache2, Apache-Mod-PHP5, and configures OwnCloud with
sqlite as a standalone server. Provides relationship hooks with NFS file
storage, MySQL Databases, and HAProxy reverse proxy charms.

# Configuration

This charm comes with different configuration options. Some of them are
mandatory, but other ones are optional. Mandatory options include:

`domain`: This is the domain or IP address for your ownCloud instance. If you
do not know what it is, execute `juju status` to find out the public address. If
not provided, the charm will refuse to configure.

Optional configuration options include:

`port`: This is the alias port that will be used for the ownCloud instance. It
will redirect to 443, which is the HTTPS port. It defaults to 80.

`src`: This is the source from which the package will be installed. You can
choose between `repo`, which will install it from a repository built by
ownCloud, or `source`, which will download the tarball and extract it. It
defaults to `repo`.

`customssl`: This charm provides default SSL support for ownCloud. This means
that if you do not provide a custom SSL key and certificate, a self-signed one
will be auto-generated for you. If you want to use a custom SSL certificate,
please set this option to `true`. It defaults to `false`.

`sslkey`: If `customssl` is set to true, this is the SSL key that will be used.
If not provided and `customssl` is true, the charm will refuse to configure.

`sslcert`: If `customssl` is set to true, this is the SSL cert that will be
used. If not provided and `customssl` is true, the charm will refuse to
configure.

`user`: This user will be used in case you want to do a setup of Shared
Instances. If not provided, it will be defaulted to `ownCloud`.

`password`: This password will be used in case you want to do a setup of
Shared Instances. If not provided, it will be randomly generated when a DB
relation is joined.

You can put any of this options in a config.yaml file and specify it at the
moment of deploying. Otherwise, you can change them by executing:

    juju set owncloud [option]=[value]

# Usage

## Standalone Instance

First, bootstrap your environment:

    juju bootstrap

Then, deploy ownCloud by executing the following command:
    
    juju deploy owncloud

Finally, expose the service:

    juju expose owncloud

Access OwnCloud service directly, and complete the setup, providing user
credentials and initializing sqlite database.

## Shared Instances

If you want to deploy shared instances, execute the following commands after
doing a Standalone Instance setup:

    juju deploy mysql
    juju add-relation mysql owncloud

    juju deploy nfs
    juju add-relation nfs owncloud

We're now done! To find out the address for your ownCloud instance, execute
`juju status` and navigate to it.

# Scale out Usage

In order to do a scalabe deploy of ownCloud, execute the following commands

    juju bootstrap
    juju deploy owncloud
    juju deploy mysql
    juju deploy haproxy
    juju add-relation mysql owncloud
    juju add-relation haproxy owncloud
    juju add-unit owncloud


# Known Limitations

If you have been using a standalone instance and want to migrate to a shared
instance, please note that adding the mysql relation will not preserve the file
structure in the database. This means that your file listing will not be
available. Make sure to have this in mind when doing the migration.

Also, if you leave the `customssl` option set to false or provide a self-signed
SSL certificate, ownCloud will throw a WebDAV error after creating the admin
username and password. Ignore this error as it does not affect the working of
ownCloud (it is silently fixed), and enter your website again.

Finally, if port is different than 80, it looks like the instance throws an SSL
error when connecting. Recommendation is to set the `port` value to 80 to avoid
problems.

#TODO
Genericize shared-fs-relation-* for non-nfs shared-fs providers
