#!/usr/bin/env python3
# The format of this test was shamelessly ripped from Cory Johns awesome
# tests that resemble nosetests in the Apache Allura charm. <3
# http://bazaar.launchpad.net/~johnsca/charms/precise/apache-allura/

import amulet
import requests


class TestDeploy(object):

    def __init__(self, time=2500):
        # Attempt to load the deployment topology from a bundle.
        self.deploy = amulet.Deployment()

        # If something errored out, attempt to continue by
        # manually specifying a standalone deployment
        self.deploy.add('owncloud')
        self.deploy.add('mysql')
        self.deploy.add('nfs')
        self.deploy.configure('owncloud', {'user': 'tom',
                                           'password': 'swordfish'})
        self.deploy.relate('owncloud:db', 'mysql:db')
        self.deploy.relate('owncloud:shared-fs', 'nfs:nfs')
        self.deploy.expose('owncloud')

        try:
            self.deploy.setup(time)
            self.deploy.sentry.wait(time)
        except:
            amulet.raise_status(amulet.FAIL, msg="Environment standup timeout")
        sentry = self.deploy.sentry

        self.domain = sentry.unit['owncloud/0'].info['public-address']
        self.mysql_unit = self.deploy.sentry.unit['mysql/0']
        self.nfs_unit = self.deploy.sentry.unit['nfs/0']
        self.app_unit = self.deploy.sentry.unit['owncloud/0']

    def run(self):
        for test in dir(self):
            if test.startswith('test_'):
                getattr(self, test)()

    def test_standalone_http(self):

        # r = requests.get("http://%s/" % domain, data, headers=h)
        r = requests.get("https://%s/index.php" % self.domain, verify=False)
        r.raise_for_status()

        search_string = 'web services under your control'
        if r.text.find(search_string) is -1:
            amulet.raise_status(amulet.FAIL, msg="Unable to verify login page")

    def test_database_relationship(self):

        sent_config = self.mysql_unit.relation('db', 'owncloud:db')
        # I'm sorry this is gross, and repeats itself. This is a dirty hack
        # around owncloud not normalizing quotations, and field names.
        try:
            filepath = "/var/www/owncloud/config/autoconfig.php"
            cfg = self.deploy.sentry.unit['owncloud/0'].file_contents(filepath)
            cfg_to_check = {'dbuser': 'user', 'dbpass': 'password',
                            'dbhost': 'host'}
            for c_key, j_key in cfg_to_check.items():
                if cfg.find('"%s" => "%s"' % (c_key, sent_config[j_key])) == -1:
                    amulet.raise_status(amulet.FAIL,
                                        msg="Unable to validate db cfg %s" % j_key)
        except:
            filepath = "/var/www/owncloud/config/config.php"
            cfg = self.deploy.sentry.unit['owncloud/0'].file_contents(filepath)
            cfg_to_check = {'dbuser': 'user', 'dbpassword': 'password',
                            'dbhost': 'host'}
            for c_key, j_key in cfg_to_check.items():
                if cfg.find("'%s' => '%s'" % (c_key, sent_config[j_key])) == -1:
                    amulet.raise_status(amulet.FAIL,
                                        msg="Unable to validate db cfg %s" % j_key)

    def term_search(self, output, term):
        if output.find(term) == -1:
            amulet.raise_status(amulet.FAIL,
                                msg="Unable to validate NFS config %s" % term)

    def test_nfs_relationship(self):
        # Cache Relationship details
        nfs_relation = self.nfs_unit.relation('nfs', 'owncloud:shared-fs')

        # Leverage Amulet's exception handling if directory doesnt exist
        # to check for the directory, as a cheap quick fail test.
        try:
            self.app_unit.directory('/var/lib/owncloud')
        except:
            amulet.raise_status(amulet.FAIL, msg="NFS Directory not found")

        #Fetch the contents of mtab for data validation
        mtab_contents = self.app_unit.file_contents('/etc/mtab')

        self.term_search(mtab_contents, nfs_relation['private-address'])
        self.term_search(mtab_contents, nfs_relation['fstype'])
        self.term_search(mtab_contents, nfs_relation['mountpoint'])
        self.term_search(mtab_contents, nfs_relation['options'])

    def test_nfs_write_pipeline(self):
        # Validate file write pipeline
        #Build a $block_size file, and ship it via NFS
        cmd_builder = "dd if=/dev/zero of=/var/lib/owncloud/amulet-file-test bs=8M count=1"
        self.app_unit.run(cmd_builder)

        filepath = '/srv/data/relation-sentry/amulet-file-test'
        file_test = self.nfs_unit.file(filepath)
        if file_test['size'] < 8000000:
            amulet.raise_status(amulet.FAIL, 'File size constraint not met')


if __name__ == '__main__':
    runner = TestDeploy()
    runner.run()
